# -*- coding: UTF-8 -*-
import json


class YelpError(Exception):

    def __init__(self, code, msg, response):
        self.code = code
        self.msg = msg

        self.id = response['error']['id']
        self.text = response['error']['text']


class AreaTooLarge(YelpError):
    pass


class BadCategory(YelpError):
    pass


class BusinessUnavailable(YelpError):
    pass


class ExceededReqs(YelpError):
    pass


class InternalError(YelpError):
    pass


class InvalidOAuthCredentials(YelpError):
    pass


class InvalidOAuthUser(YelpError):
    pass


class InvalidSignature(YelpError):
    pass


class MissingParameter(YelpError):
    pass


class MultipleLocations(YelpError):
    pass


class SSLRequired(YelpError):
    pass


class UnavailableForLocation(YelpError):
    pass


class UnspecifiedLocation(YelpError):
    pass


class InvalidParameter(YelpError):

    def __init__(self, code, msg, response):
        super(InvalidParameter, self).__init__(code, msg, response)
        self.text += ': ' + response['error']['field']


class ErrorHandler(object):

    _error_map = {
        'AREA_TOO_LARGE': AreaTooLarge,
        'BAD_CATEGORY': BadCategory,
        'BUSINESS_UNAVAILABLE': BusinessUnavailable,
        'EXCEEDED_REQS': ExceededReqs,
        'INTERNAL_ERROR': InternalError,
        'INVALID_OAUTH_CREDENTIALS': InvalidOAuthCredentials,
        'INVALID_OAUTH_USER': InvalidOAuthUser,
        'INVALID_SIGNATURE': InvalidSignature,
        'INVALID_PARAMETER': InvalidParameter,
        'MISSING_PARAMETER': MissingParameter,
        'MULTIPLE_LOCATIONS': MultipleLocations,
        'SSL_REQUIRED': SSLRequired,
        'UNAVAILABLE_FOR_LOCATION': UnavailableForLocation,
        'UNSPECIFIED_LOCATION': UnspecifiedLocation
    }

    def raise_error(self, error):
        try:
            error_msg = error.read()
            response = json.loads(error_msg.decode('UTF-8'))
            raise self._error_map[response['error']['id']](
                error.code,
                error.msg,
                response
            )
        except (KeyError, ValueError):
            raise error




# -*- coding: UTF-8 -*-
""" Yelp v3 API see: https://www.yelp.com/developers/documentation/v3/ """

import json

from six.moves import urllib as urllib
from yelp3.errors import ErrorHandler

API_HOST = 'api.yelp.com'

BUSINESS_SEARCH_PATH     = '/v3/businesses/search'
PHONE_SEARCH_PATH        = '/v3/businesses/search/phone'
TRANSACTION_SEARCH_PATH  = '/v3/transactions/{transaction_type}/search'
BUSINESS_PATH            = '/v3/businesses/{id}'
REVIEWS_PATH             = '/v3/businesses/{id}/reviews'
AUTOCOMPLETE_PATH        = '/v3/autocomplete'


class Client(object):

    def __init__(self, access_token, debug=False):
        self.access_token = access_token
        self._error_handler = ErrorHandler()
        self.debug = debug

    def business_search(self, **url_params):
        return self._make_request(
            path = BUSINESS_SEARCH_PATH,
            url_params = dict(**url_params)
        )

    def phone_search(self, **url_params):
        return self._make_request(
            path = PHONE_SEARCH_PATH,
            url_params = dict(**url_params)
        )

    def transaction_search(self, transaction_type, **url_params):
        return self._make_request(
            path = TRANSACTION_SEARCH_PATH.format(
                transaction_type = transaction_type
            ),
            url_params = dict(**url_params)
        )


    def business(self, _id, **url_params):
        return self._make_request(
            path = BUSINESS_PATH.format(
                id = _id
            ),
            url_params = dict(**url_params)
        )

    def review(self, _id, **url_params):
        return self._make_request(
            path = REVIEWS_PATH.format(
                id = _id
            ),
            url_params = dict(**url_params)
        )

    def autocomplete(self, **url_params):
        return self._make_request(
            path = AUTOCOMPLETE_PATH,
            url_params = dict(**url_params)
        )
    
    @staticmethod
    def _filter_dict(old_dict, cb):
        """ Returns a filtered dictionary based on the result of `cb(key)`. """
        return {k: v for k, v in old_dict.items() if cb(k)}
    
    def _make_request(self, path, url_params={}):

        # filter out parameters that equal None
        url_params = Client._filter_dict(url_params, lambda k: url_params[k] is not None)

        url = 'https://{0}{1}?'.format(
            API_HOST,
            urllib.parse.quote(path.encode('utf-8'))
        ) + urllib.parse.urlencode(url_params)

        request = urllib.request.Request(
            url,
            headers = {
                "Authorization": "Bearer {0}".format(self.access_token)
            }
        )

        try:
            conn = urllib.request.urlopen(request)
        except urllib.error.HTTPError as error:
            self._error_handler.raise_error(error)

        try:
            data = conn.read()
            response = json.loads(data.decode('utf-8'))

            if self.debug:
                print("\n> " + request.get_full_url())
                print("")
                print(str(conn.info()))

        finally:
            conn.close()

        return response




from setuptools import find_packages
from setuptools import setup

setup(
    name='yelp3',

    version='1.0.1',

    description='Python implementation of the Yelp V3 Fusion API',

    url='https://github.com/tapslash/yelp3-python',

    author='David Hargat',

    author_email='davidmhargat@gmail.com',

    license='MIT',

    keywords='yelp',

    packages=find_packages(),

    install_requires=['six'],
)
